﻿unit uMain;

interface

uses
  Winapi.Windows, Winapi.Messages, Generics.Collections, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  System.NetEncoding, Vcl.StdCtrls, RzCmboBx, uSM4,
  Vcl.Mask, RzEdit;

type
  TMainForm = class(TForm)
    LabelBlockMode: TLabel;
    LabelPaddingmode: TLabel;
    ComboBoxCipherMode: TComboBox;
    ComboBoxPaddingmode: TComboBox;
    LabelStr: TLabel;
    LabelIV: TLabel;
    LabelEncryptedStr: TLabel;
    LabelDecryptedStr: TLabel;
    EditPassword: TEdit;
    EditIV: TEdit;
    EditDecryptedStr: TEdit;
    EditEncryptedStr: TEdit;
    EditStr: TEdit;
    LabelPassword: TLabel;
    ButtonEncrypt: TButton;
    ButtonDecrypt: TButton;
    ComboBoxStr: TRzComboBox;
    ComboBoxPassword: TRzComboBox;
    ComboBoxIV: TRzComboBox;
    ComboBoxEncryptedStr: TRzComboBox;
    ComboBoxDecryptedStr: TRzComboBox;
    LabelStrCnt: TLabel;
    LabelPasswordCnt: TLabel;
    LabelIVCnt: TLabel;
    LabelEncryptedStrCnt: TLabel;
    LabelDecryptedStrCnt: TLabel;
    EditSrc: TEdit;
    ButtonToHex: TButton;
    ButtonToBase64: TButton;
    EditDest: TEdit;
    RzEdit1: TRzEdit;
    Label1: TLabel;
    ButtonFromHex: TButton;
    ButtonFramBase64: TButton;
    Label2: TLabel;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonConvertClick(Sender: TObject);
    procedure ButtonFromClick(Sender: TObject);
  private type
    TProc = procedure(const S: String) of object;
    Tfunc = function(): String of object;
    TFromDict = TDictionary<String, TProc>;
    TToDict   = TDictionary<String, TFunc>;
  private
    SM4: TSM4;
    CmDIct: TDictionary<String, TCipherMode>;
    PmDIct: TDictionary<String, TPaddingMode>;
    DictFromStr, DictFromPaasword, DictFromIV, DictFromEncryptedStr: TFromDict;
    DictToEncryptedStr, DictToDecryptedStr: TToDict;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses uBuffer;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  SM4 := TSM4.Create;

  EditChange(nil);
  CmDIct := TDictionary<String, TCipherMode>.Create;

  CmDIct.Add('ECB' , cmECB);
  CmDIct.Add('CBC' , cmCBC);
  CmDIct.Add('CFB' , cmCFB);
  CmDIct.Add('OFB' , cmOFB);
  CmDIct.Add('PCBC', cmPCBC);
  CmDIct.Add('CTR' , cmCTR);

  PmDIct := TDictionary<String, TPaddingMode>.Create;
  PmDIct.Add('ZERO'      , pmZERO);
  PmDIct.Add('PKCS5'     , pmPKCS5);
  PmDIct.Add('PKCS7'     , pmPKCS7);
  PmDIct.Add('ISO10126'  , pmISO10126);
  PmDIct.Add('ANSIX923'  , pmANSIX923);
  PmDIct.Add('OneAndZero', pmOneAndZero);

  DictFromStr.Add('String', SM4.SrcBuffer.FromString);
  DictFromStr.Add('Hex'   , SM4.SrcBuffer.FromHexString);
  DictFromStr.Add('Base64', SM4.SrcBuffer.FromBase64String);

  DictFromPaasword.Add('String', SM4.KeyBuffer.FromString);
  DictFromPaasword.Add('Hex'   , SM4.KeyBuffer.FromHexString);
  DictFromPaasword.Add('Base64', SM4.KeyBuffer.FromBase64String);

  DictFromIV.Add('String', SM4.IVBuffer.FromString);
  DictFromIV.Add('Hex'   , SM4.IVBuffer.FromHexString);
  DictFromIV.Add('Base64', SM4.IVBuffer.FromBase64String);

  DictFromEncryptedStr.Add('Hex'   , SM4.SrcBuffer.FromHexString);
  DictFromEncryptedStr.Add('Base64', SM4.SrcBuffer.FromBase64String);

  DictToEncryptedStr.Add('Hex'   , SM4.SrcBuffer.ToHexString);
  DictToEncryptedStr.Add('Base64', SM4.SrcBuffer.ToBase64String);

  DictToDecryptedStr.Add('String', SM4.SrcBuffer.ToString);
  DictToDecryptedStr.Add('Hex'   , SM4.SrcBuffer.ToHexString);
  DictToDecryptedStr.Add('Base64', SM4.SrcBuffer.ToBase64String);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  CmDIct.Free;
  PmDIct.Free;
  SM4.Free;
end;

procedure TMainForm.ButtonConvertClick(Sender: TObject);
begin
  with TBuffer.Create do
  begin
    FromString(EditSrc.Text);
    if Sender = ButtonToHex then
      EditDest.Text := ToHexString
    else
      EditDest.Text := ToBase64String;
    Free;
  end;
end;

procedure TMainForm.ButtonFromClick(Sender: TObject);
begin
  with TBuffer.Create do
  begin
    if Sender = ButtonFromHex then
      FromHexString(EditDest.Text)
    else
      FromBase64String(EditDest.Text);
    EditSrc.Text := ToString;
    Free;
  end;
end;

procedure TMainForm.ButtonClick(Sender: TObject);
//    DictFromStr, DictFromPaasword, DictFromIV, DictFromEncryptedStr: TFromDict;
//    DictToEncryptedStr, DictToDecryptedStr: TToDict;
begin
  with SM4 do
  begin
    CipherMode  := CmDIct.Items[ComboBoxCipherMode.Text];
    PaddingMode := PmDIct.Items[ComboBoxPaddingMode.Text];

    {if ComboBoxPassword.Text = 'String' then        //key
      KeyBuffer.FromString(EditPassword.Text)
    else if ComboBoxPassword.Text = 'Hex' then
      KeyBuffer.FromHexString(EditPassword.Text)
    else if ComboBoxPassword.Text = 'Base64' then
      KeyBuffer.FromBase64String(EditPassword.Text);}
    DictFromPaasword.Items[ComboBoxPassword.Text](EditPassword.Text);

    {if ComboBoxIV.Text = 'String' then             //IV
      IVBuffer.FromString(EditIV.Text)
    else if ComboBoxIV.Text = 'Hex' then
      IVBuffer.FromHexString(EditIV.Text)
    else if ComboBoxStr.Text = 'Base64' then
      IVBuffer.FromBase64String(EditStr.Text);}
    DictFromIV.Items[ComboBoxIV.Text](EditIV.Text);

    if Sender = ButtonEncrypt then
    begin
      {if ComboBoxStr.Text = 'String' then           //Source
        SrcBuffer.FromString(EditStr.Text)
      else if ComboBoxStr.Text = 'Hex' then
        SrcBuffer.FromHexString(EditStr.Text)
      else if ComboBoxStr.Text = 'Base64' then
        SrcBuffer.FromBase64String(EditStr.Text);}
      DictFromStr.Items[ComboBoxStr.Text](EditStr.Text);

      Encrypt;

      {if ComboBoxEncryptedStr.Text = 'Hex' then
        EditEncryptedStr.Text := DestBuffer.ToHexString
      else if ComboBoxEncryptedStr.Text = 'Base64' then
        EditEncryptedStr.Text := DestBuffer.ToBase64String;}
      EditEncryptedStr.Text :=  DictToEncryptedStr.Items[ComboBoxEncryptedStr.Text]();
    end
    else
    begin
      if ComboBoxEncryptedStr.Text = 'String' then      //Source
        SrcBuffer.FromString(EditEncryptedStr.Text)
      else if ComboBoxEncryptedStr.Text = 'Hex' then
        SrcBuffer.FromHexString(EditEncryptedStr.Text)
      else if ComboBoxEncryptedStr.Text = 'Base64' then
        SrcBuffer.FromBase64String(EditEncryptedStr.Text);

      Decrypt;

      if ComboBoxDecryptedStr.Text = 'String' then
        EditDecryptedStr.Text := DestBuffer.ToString
      else if ComboBoxDecryptedStr.Text = 'Hex' then
        EditDecryptedStr.Text := DestBuffer.ToHexString
      else if ComboBoxDecryptedStr.Text = 'Base64' then
        EditDecryptedStr.Text := DestBuffer.ToBase64String;
    end;
  end;
end;

procedure TMainForm.EditChange(Sender: TObject);
begin
  if Sender = EditStr then
    LabelStrCnt.Caption := Length(EditStr.Text).ToString
  else if Sender = EditPassword then
    LabelPasswordCnt.Caption := Length(EditPassword.Text).ToString
  else if Sender = EditIV then
    LabelIVCnt.Caption := Length(EditIV.Text).ToString
  else if Sender = EditEncryptedStr then
    LabelEncryptedStrCnt.Caption := Length(EditEncryptedStr.Text).ToString
  else if Sender = EditDecryptedStr then
    LabelDecryptedStrCnt.Caption := Length(EditDecryptedStr.Text).ToString
  else if Sender = nil then
  begin
    LabelStrCnt.Caption := Length(EditStr.Text).ToString;
    LabelPasswordCnt.Caption := Length(EditPassword.Text).ToString;
    LabelIVCnt.Caption := Length(EditIV.Text).ToString;
    LabelEncryptedStrCnt.Caption := Length(EditEncryptedStr.Text).ToString;
    LabelDecryptedStrCnt.Caption := Length(EditDecryptedStr.Text).ToString;
  end;
end;

end.
